@@ -22,7 +22,6 @@ an explicit registration.
 from typing import Any, Dict, Iterable, List, Optional, Type
 
 from django.urls import converters
-from render_static.exceptions import PlaceholderNotFound
 
 __all__ = [
     'register_converter_placeholder',
@@ -32,6 +31,12 @@ __all__ = [
     'resolve_unnamed_placeholders'
 ]
 
+# DO NOT EXPAND THIS LIST LIGHTLY - for URLs with large numbers of arguments, say 5 where all these
+# defaults are tried that would be an O(n^5) operation, where n is the number of defaults.
+# URLs with large numbers of arguments should be rare - but there is a complexity check that will
+# throw an exception when a maximum number of tries is reached
+ALWAYS_TRY_THESE = [0, 'a', 1, 'A', 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa']
+
 converter_placeholders: Dict[Type, List] = {
     converters.IntConverter: [0],
     converters.PathConverter: ['a'],
@@ -96,7 +101,7 @@ def register_unnamed_placeholders(
 ) -> None:
     """
     Register a list of placeholders for a url_name and optionally an app_name that takes unnamed
-    arguments.
+    arguments. The list indices should correspond to the argument order as passed to reverse.
 
     :param url_name: The name of the url path to register the placeholders for
     :param placeholders: The list of placeholders to use
@@ -137,62 +142,41 @@ def resolve_placeholders(
     if app_name:
         placeholders.extend(app_variable_placeholders.get(app_name, {}).get(var_name, []))
     placeholders.extend(variable_placeholders.get(var_name, []))
-
-    if not placeholders:
-        lookup = {
-            'parameter': var_name,
-            'converter': str(converter),
-            'app_name': app_name
-        }
-        raise PlaceholderNotFound(
-            f'No placeholders are registered for any of the lookup parameters: {lookup}'
-        )
-
-    return placeholders
+    return placeholders + [always for always in ALWAYS_TRY_THESE if always not in placeholders]
 
 
 def resolve_unnamed_placeholders(
         url_name: str,
-        app_name: Optional[str] = None
+        nargs: int,
+        app_name: Optional[str] = None,
 ) -> Iterable:
     """
     Resolve placeholders to use for a url with unnamed parameters based on the url name and
     optionally the app_name.
 
     :param url_name: The name of the URL to search for
+    :param nargs: The number of arguments for this url
     :param app_name: The optional app_name to search for
-    :return: A list of lists of placeholders to try
+    :return: A list of lists of placeholders to try, where the outer list is indexed by argument
+        index
     """
 
-    placeholders = []
-    if app_name:
-        placeholders.extend(app_unnamed_placeholders.get(app_name, {}).get(url_name, []))
-    placeholders.extend(unnamed_placeholders.get(url_name, []))
+    placeholders: List[List[object]] = [[] for arg in range(0, nargs)]
 
-    if not placeholders:
-        lookup = {
-            'url_name': url_name,
-            'app_name': app_name
-        }
-        raise PlaceholderNotFound(
-            f'No unnamed placeholders are registered for any of the lookup parameters: {lookup}'
-        )
+    def add_candidates(candidates: List[List[object]]) -> None:
+        for candidate in candidates:
+            if len(candidate) == nargs:
+                for idx, arg in enumerate(candidate):
+                    placeholders[idx].append(arg)
+    if app_name:
+        add_candidates(app_unnamed_placeholders.get(app_name, {}).get(url_name, []))
+    add_candidates(unnamed_placeholders.get(url_name, []))
+
+    for arg in placeholders:
+        arg.extend([always for always in ALWAYS_TRY_THESE if always not in arg])
 
     return placeholders
 
 
 register_variable_placeholder('app_label', 'site', app_name='admin')
 register_variable_placeholder('app_label', 'auth', app_name='admin')
-
-# Register some common placeholders
-register_variable_placeholder(var_name='key', placeholder='a')
-register_variable_placeholder(var_name='key', placeholder=0)
-
-# allauth
-register_variable_placeholder(var_name='uidb36', placeholder=0)
-
-# DRF
-register_variable_placeholder(var_name='format', placeholder='a')
-
-register_variable_placeholder(var_name='pk', placeholder=0)
-register_variable_placeholder(var_name='pk', placeholder='a')
